#!/usr/bin/python3
# -*- coding: UTF-8 -*-

import gi
gi.require_version("Gtk","3.0")
from gi.repository import Gtk as gtk
from gi.repository import GObject as gobject
from gi.repository import GLib as glib
from gi.repository import Gio as gio

import os
import os.path
import sys
import string
import time
import subprocess
import gettext
import dbus
import dbus.mainloop.glib
import logging
import signal
import errno

t = gettext.translation('nfs-upgrade-tool','/usr/share/locale')
_ = t.gettext

class TrayIndicator(gobject.GObject):
    def __init__(self):
        self.tray = gtk.StatusIcon.new_from_file('/usr/share/nfs-upgrade-tool/data/icon.png')
        self.tray.set_has_tooltip(True)
        self.tray.set_tooltip_text("nfs upgrade tool")
        self.tray.set_name("nfs-upgrade-tool")
        self.tray.set_title("nfs-upgrade-tool")
        self.tray.connect("activate",self.on_activate)
        self.tray.connect("popup-menu",self.build_menu)

        #set the contextmenu
        self.menu = gtk.Menu()
        self.add_menu_item(_("_Check update"),self.item_check_update_cb)
        self.update_item = self.add_menu_item(_("_Update now"),self.item_udpate_now_cb)
        self.update_item.set_sensitive(False)
        self.auto_item = gtk.CheckMenuItem(label=_("_Auto update"),use_underline=True)
        self.auto_item.connect("toggled",self.item_set_auto_cb)
        self.menu.append(self.auto_item)
        self.add_menu_item(_("_Quit"),self.quit)

        self.dbus = dbus.SystemBus()
        try:
            update_bus = self.dbus.get_object('com.nfs.update','/')
            self.dbus_iface = dbus.Interface(update_bus,dbus_interface='com.nfs.update')
            self.dbus_iface.connect_to_signal("Finished",self.dbus_on_finished)
            self.check_update = self.dbus_iface.get_dbus_method("checkUpdate")
        except:
            logging.error("dbus:com.nfs.update is missing.")
            sys.exit(1)

        self.pid = 0
        self.appinfo = gio.DesktopAppInfo.new_from_filename("/usr/share/applications/nfs-upgrade-tool.desktop")
        glib.idle_add(self.do_idle,None)

    def do_idle(self,user_data=None):
        self.check_update()
        self.check_updating()
        self.update_tray_status()
        self.tray.set_visible(True)
        return False

    def check_updating(self):
        dbus_is_updating = self.dbus_iface.get_dbus_method("isUpdating")
        is_updating = dbus_is_updating()
        self.update_item.set_sensitive(not is_updating)

    def check_automatic(self):
        dbus_is_automatic = self.dbus_iface.get_dbus_method("isAutomatic")
        is_auto = dbus_is_automatic()
        self.auto_item.set_active(is_auto)

    def add_menu_item(self,label,callback):
        item = gtk.MenuItem(label=label,use_underline=True)
        #item.set_use_underline(True)
        item.connect("activate",callback)
        self.menu.append(item)
        return item

    def item_check_update_cb(self,item = None):
        self.check_update()

    def item_set_auto_cb(self,item):
        active = item.get_active()
        dbus_set_automatic = self.dbus_iface.get_dbus_method("setAutomatic")
        a = dbus_set_automatic(active)
        if not a:
            #set failed
            item.set_active(not active)

    def dbus_on_finished(self,action,result,msg=None):
        #print("action = %s, result = %d" % (action, result))
        if action == 'UPDATE_LIST':
            self.update_tray_status()
        elif action == 'IS_AUTOMATIC':
            self.check_automatic()
        elif action == 'UPDATE' or action == 'IS_UPDATING':
            self.check_updating()

    def update_tray_status(self):
        list_len = 0
        get_update_pkgs = self.dbus_iface.get_dbus_method("getUpdatePkgs")
        updates = get_update_pkgs()
        list_len = len(updates)
        if list_len > 0:
            self.update_item.set_sensitive(True)
        else:
            self.update_item.set_sensitive(False)
        self.tray.set_tooltip_text(_("%d packages need update") % list_len)

    def open_client(self):
        has_pid = False
        if self.pid > 0:
            try:
                os.kill(self.pid, 0)
            except OSError as err:
                if err.errno == errno.ESRCH:
                    # ESRCH == No such process
                    has_pid = False
                elif err.errno == errno.EPERM:
                    # EPERM clearly means there's a process to deny access to
                    has_pid = True
                else:
                    # According to "man 2 kill" possible error values are
                    # (EINVAL, EPERM, ESRCH)
                    raise
            else:
                has_pid = True
        if has_pid:
            print("The client has launched.")
            return False

        context = gio.AppLaunchContext()
        context.connect("launched",self.client_launched,self)
        launched = self.appinfo.launch(None,context)
        return launched

    def client_launched(self,context,info,platform_data,user_data):
        #print(platform_data['pid'])
        self.pid = platform_data['pid']

    def on_activate(self,status_icon):
        self.open_client()

    def item_udpate_now_cb(self,item = None):
        self.update_item.set_sensitive(False)
        dbus_update_now = self.dbus_iface.get_dbus_method("updateNow")
        result = dbus_update_now()

    def build_menu(self,status_icon,button,time):
        #print("build menu: button = %d" % button)
        if (button == 3):
            self.menu.show_all()
            self.menu.popup(None,None,status_icon.position_menu,status_icon,button,time)

    def quit(self,item = None):
        sys.exit(0)

def main():
    TrayIndicator()
    gtk.main()

if __name__ == '__main__':
    dbus.mainloop.glib.DBusGMainLoop(set_as_default=True)
    main()
    sys.exit(0)